/********************** (C) COPYRIGHT 2007-2009 RAISONANCE ********************
* File Name          :  Application.c
* Author             :
* Date First Issued  :
* Description        :  Circle_App CircleOS application template.
* Revision           :
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "circle_api.h"
#include "hw_config.h"
#include "stm32f10x_lib.h"
#include "stm32f10x_usart.h"
#include <string.h>

/* Private defines -----------------------------------------------------------*/

// The following should be the minimal CircleOS version needed by your application
#define MAX_TOGGLE_COUNT (0x8)   //this tells the speed of the toggle      
#define NEEDEDVERSION "V 1.5"

/* Private variables ---------------------------------------------------------*/
u32 gNext = 0;
char gCommand[1100] = {""};

/* Private functions ---------------------------------------------------------*/
enum MENU_code MsgVersion(void);

/* Public variables ----------------------------------------------------------*/
const char Application_Name[8+1] = {"IrDA-LCD"};    // Max 8 characters

void IRDA_USART_Configuration( void ) ;
void IRDA_GPIO_Configuration(void) ; 

void IRDA_GPIO_Configuration(void)
    {
    GPIO_InitTypeDef GPIO_InitStructure;

    /* Configure USART1 Tx (PA.9)as alternate function push-pull */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* Configure USART1 Rx PA.10 as input floating */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    /* Configure GPIO_D Pin 8 = CS Irda */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    }

void IRDA_USART_Configuration( void ) 
{
    USART_InitTypeDef USART_InitStructure;

//    USART_InitStructure.USART_BaudRate = 115200;
    USART_InitStructure.USART_BaudRate = 76800;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No ;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

    /* Configure the USART1 */
    USART_Init(USART1, &USART_InitStructure);

    /* Enable the USART1 */
    USART_Cmd(USART1, ENABLE);

    /* Set the USART1 prescaler */
    USART_SetPrescaler(USART1, 0x1);

    /* Configure the USART1 IrDA mode */
    USART_IrDAConfig(USART1, USART_IrDAMode_Normal);

    /* Enable the USART1 IrDA mode */
    USART_IrDACmd(USART1, ENABLE);

    // CS Irda  = 0
    GPIO_ResetBits(GPIOD, GPIO_Pin_8);
}

/*******************************************************************************
* Function Name  : Application_Ini
* Description    : Initialization function of Circle_App. This function will
*                  be called only once by CircleOS.
* Input          : None
* Return         : MENU_CONTINUE_COMMAND
*******************************************************************************/
enum MENU_code Application_Ini ( void )
{
/*    if(strcmp(UTIL_GetVersion(), NEEDEDVERSION) < 0)
        {
        return MsgVersion();
        }*/

    //Enable clock
    RCC_APB2PeriphClockCmd(
    RCC_APB2Periph_GPIOD | 
    RCC_APB2Periph_AFIO | 
    RCC_APB2Periph_USART1, ENABLE);       
    UTIL_SetPll(SPEED_VERY_HIGH);
    IRDA_GPIO_Configuration();
    IRDA_USART_Configuration();

    return MENU_CONTINUE_COMMAND;
}

/*******************************************************************************
* Function Name  : Application_Handler
* Description    : Management of the Circle_App.
*
* Input          : None
* Return         : MENU_CONTINUE
*******************************************************************************/
enum MENU_code Application_Handler ( void )
{
    int i, count;
    int Xpos, Ypos, Xend, Yend;
//    u8 sTxData;
    u8 sRxData;
    u32 ColorNo;
    char *adr;
    char sXpos[4] = "";
    char sYpos[4] = "";
    char sXend[4] = "";
    char sYend[4] = "";
    char sMessage[32] = "";

    // for debug
/*    sTxData = 'A';
    for(i=0; i<10; i++) {
        // send the character
        USART_SendData(USART1, sTxData);
        while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)
            { ; }
        sTxData++;
    }
    DRAW_DisplayString(10,75,"* Test TX OK *",15);*/

    if (USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == SET) {
        do {
            count = 0;
            while (USART_GetFlagStatus(USART1, USART_FLAG_RXNE) != SET) {
                count++;
                if (count > 800000) {
//                        DRAW_DisplayString(10,75,"break",6);  // for debug
                        break;
                }
            }

            if (count > 800000) {
                break;
            }
            else {
                gCommand[gNext] = USART_ReceiveData(USART1);
                gNext++;
            }
        } while (gCommand[gNext-1] != NULL);
        
        if (gCommand[gNext-1] == NULL) {
//            DRAW_DisplayString(0,110,gCommand,strlen(gCommand)+1);  // for debug            
            gNext=0;

            // ClearLCD()
            if (strstr(gCommand, "ClearLCD") != NULL) {
                LCD_FillRect(0,0,128,128, RGB_WHITE);
//                LCD_DrawRect(0,0,128,128, RGB_WHITE); // anather code
            }

            // Xpos
            adr = strstr(gCommand, "(");
//            DRAW_DisplayString(10,35,adr,strlen(adr)+1); // for debug
            strncpy(sXpos, adr+1, 3);
            sXpos[3] = 0;
//            DRAW_DisplayString(10,50,sXpos,4);    // for debug
            Xpos = atoi(sXpos) & 0x7F;

            // Ypos
            adr = strstr(gCommand, ",");
            strncpy(sYpos, adr+1, 3);
            sYpos[3] = 0;
            Ypos = atoi(sYpos) & 0x7F;

            // Message
            adr = strstr(adr+1, ",");
            strncpy(sMessage, adr+1, strlen(adr));
            sMessage[strlen(sMessage)-1] = NULL;    // "(" -> NULL

            // Print()
            if (strstr(gCommand, "Print") != NULL) {
                DRAW_DisplayString(Xpos,Ypos,sMessage,strlen(sMessage)+1);
            }

            if ((strstr(gCommand, "RED") != 0)) {
                ColorNo = RGB_RED;
            }
            else if ((strstr(gCommand, "GREEN") != 0)) {
                ColorNo = RGB_GREEN;
            }
            else if ((strstr(gCommand, "BLUE") != 0)) {
                ColorNo = RGB_BLUE;
            }
            else {
                ColorNo = RGB_BLACK;
            }

            // Pixel()
            if (strstr(gCommand, "Pixel") != NULL) {
                count = Xpos;   // Xpos=Pixel count
                for(i=0; i<count; i++) {
                    adr = strstr(sMessage, ",");
                    strncpy(sMessage, adr+1, strlen(adr)); 
                    strncpy(sXpos, sMessage, 3);
                    sXpos[3] = 0;
                    Xpos = atoi(sXpos) & 0x7F;

                    adr = strstr(sMessage, ",");
                    strncpy(sMessage, adr+1, strlen(adr)); 
                    strncpy(sYpos, sMessage, 3);
                    sYpos[3] = 0;
                    Ypos = atoi(sYpos) & 0x7F;
                
                    LCD_DrawPixel(Xpos,Ypos, ColorNo);
                }
            }
        
            // Line()
            if (strstr(gCommand, "Line") != NULL) {
                // Xend
                strncpy(sXend, sMessage, 3);
                sXend[3] = 0;
//                DRAW_DisplayString(10,80,sXend,4);    // for debug
                Xend = atoi(sXend) & 0x7F;

                // Yend
                strncpy(sYend, sMessage+4, 3);
                sYend[3] = 0;
//                DRAW_DisplayString(10,95,sYend,4);    // for debug
                Yend = atoi(sYend) & 0x7F;
            
                DRAW_Line(Xpos,Ypos,Xend,Yend, ColorNo);
            }
        }
     }

    // If button is pushed exit 
    if (  BUTTON_GetState() == BUTTON_PUSHED )
    {
        BUTTON_WaitForRelease();   
        BUTTON_SetMode  ( BUTTON_ONOFF_FORMAIN );
        LED_Set ( LED_GREEN,LED_OFF);
        LED_Set ( LED_RED,LED_OFF);
        LCD_SetRotateScreen(1);
        MENU_ClearCurrentCommand();
        DRAW_SetDefaultColor ();
        DRAW_Clear();
        POINTER_SetMode ( POINTER_ON );
        return MENU_LEAVE;
    }

    return MENU_CONTINUE;
}

/*******************************************************************************
* Function Name  : MsgVersion
* Description    : Display the current CircleOS version and the version needed
*                  exit to main menu after 4 secondes
*
* Input          : None
* Return         : MENU_CONTINUE
*******************************************************************************/
enum MENU_code MsgVersion(void)
    {
    int hh,mm,ss,ss2;

    DRAW_DisplayString(5,60,"CircleOS",17);
    DRAW_DisplayString(80,60,UTIL_GetVersion(),3);
    DRAW_DisplayString(5,34,NEEDEDVERSION,3);
    DRAW_DisplayString(40,34," required",12);

    RTC_GetTime(&hh,&mm,&ss);
    ss = ss + 4;                    // 4 seconds
    ss = ss%60;

    do
        {
        RTC_GetTime(&hh,&mm,&ss2);
        }
    while (ss2 != ss);              // do while < 4 seconds

    return MENU_REFRESH;
    }
